@@ -33,20 +33,6 @@
 #pragma once
 
 // Must include gtest first to guard against Xlib colliding due to redefinitions of "None" and "Bool"
-
-#if defined(_MSC_VER)
-#pragma warning(push)
-/*
-    MSVC warnings 4251 and 4275 have to do with potential dll-interface mismatch
-    between library (gtest) and users. Since we build the gtest library
-    as part of the test build we know that the dll-interface will match and
-    can disable these warnings.
- */
-#pragma warning(disable : 4251)
-#pragma warning(disable : 4275)
-#endif
-
-// GTest and Xlib collide due to redefinitions of "None" and "Bool"
 #if defined(VK_USE_PLATFORM_XLIB_KHR)
 #pragma push_macro("None")
 #pragma push_macro("Bool")
@@ -60,10 +46,18 @@
 #endif
 #endif
 
-// Use the NDK's header on Android
 #include "gtest/gtest.h"
 
-#include "test_util.h"
+#include "util/builder_defines.h"
+#include "util/dispatch_table.h"
+#include "util/dynamic_library_wrapper.h"
+#include "util/env_var_wrapper.h"
+#include "util/equality_helpers.h"
+#include "util/folder_manager.h"
+#include "util/functions.h"
+#include "util/manifest_builders.h"
+#include "util/test_defines.h"
+#include "util/vulkan_object_wrappers.h"
 
 #include "shim/shim.h"
 
@@ -71,14 +65,7 @@
 
 #include "layer/test_layer.h"
 
-#include FRAMEWORK_CONFIG_HEADER
-
-// Useful defines
-#if COMMON_UNIX_PLATFORMS
-#define HOME_DIR "/home/fake_home"
-#define USER_LOCAL_SHARE_DIR HOME_DIR "/.local/share"
-#define ETC_DIR "/etc"
-#endif
+#include "generated/vk_result_to_string_helper.h"
 
 // handle checking
 template <typename T>
@@ -125,152 +112,26 @@ void handle_assert_equal(size_t count, T left[], T right[]) {
     }
 }
 
-// VulkanFunctions - loads vulkan functions for tests to use
-
-struct VulkanFunctions {
-#if !defined(APPLE_STATIC_LOADER)
-    LibraryWrapper loader;
-#endif
-    // Pre-Instance
-    PFN_vkGetInstanceProcAddr vkGetInstanceProcAddr = nullptr;
-    PFN_vkEnumerateInstanceExtensionProperties vkEnumerateInstanceExtensionProperties = nullptr;
-    PFN_vkEnumerateInstanceLayerProperties vkEnumerateInstanceLayerProperties = nullptr;
-    PFN_vkEnumerateInstanceVersion vkEnumerateInstanceVersion = nullptr;
-    PFN_vkCreateInstance vkCreateInstance = nullptr;
-
-    // Instance
-    PFN_vkDestroyInstance vkDestroyInstance = nullptr;
-    PFN_vkEnumeratePhysicalDevices vkEnumeratePhysicalDevices = nullptr;
-    PFN_vkEnumeratePhysicalDeviceGroups vkEnumeratePhysicalDeviceGroups = nullptr;
-    PFN_vkGetPhysicalDeviceFeatures vkGetPhysicalDeviceFeatures = nullptr;
-    PFN_vkGetPhysicalDeviceFeatures2 vkGetPhysicalDeviceFeatures2 = nullptr;
-    PFN_vkGetPhysicalDeviceFormatProperties vkGetPhysicalDeviceFormatProperties = nullptr;
-    PFN_vkGetPhysicalDeviceFormatProperties2 vkGetPhysicalDeviceFormatProperties2 = nullptr;
-    PFN_vkGetPhysicalDeviceImageFormatProperties vkGetPhysicalDeviceImageFormatProperties = nullptr;
-    PFN_vkGetPhysicalDeviceImageFormatProperties2 vkGetPhysicalDeviceImageFormatProperties2 = nullptr;
-    PFN_vkGetPhysicalDeviceSparseImageFormatProperties vkGetPhysicalDeviceSparseImageFormatProperties = nullptr;
-    PFN_vkGetPhysicalDeviceSparseImageFormatProperties2 vkGetPhysicalDeviceSparseImageFormatProperties2 = nullptr;
-    PFN_vkGetPhysicalDeviceProperties vkGetPhysicalDeviceProperties = nullptr;
-    PFN_vkGetPhysicalDeviceProperties2 vkGetPhysicalDeviceProperties2 = nullptr;
-    PFN_vkGetPhysicalDeviceQueueFamilyProperties vkGetPhysicalDeviceQueueFamilyProperties = nullptr;
-    PFN_vkGetPhysicalDeviceQueueFamilyProperties2 vkGetPhysicalDeviceQueueFamilyProperties2 = nullptr;
-    PFN_vkGetPhysicalDeviceMemoryProperties vkGetPhysicalDeviceMemoryProperties = nullptr;
-    PFN_vkGetPhysicalDeviceMemoryProperties2 vkGetPhysicalDeviceMemoryProperties2 = nullptr;
-    PFN_vkGetPhysicalDeviceSurfaceSupportKHR vkGetPhysicalDeviceSurfaceSupportKHR = nullptr;
-    PFN_vkGetPhysicalDeviceSurfaceFormatsKHR vkGetPhysicalDeviceSurfaceFormatsKHR = nullptr;
-    PFN_vkGetPhysicalDeviceSurfacePresentModesKHR vkGetPhysicalDeviceSurfacePresentModesKHR = nullptr;
-    PFN_vkGetPhysicalDeviceSurfaceCapabilitiesKHR vkGetPhysicalDeviceSurfaceCapabilitiesKHR = nullptr;
-    PFN_vkEnumerateDeviceExtensionProperties vkEnumerateDeviceExtensionProperties = nullptr;
-    PFN_vkEnumerateDeviceLayerProperties vkEnumerateDeviceLayerProperties = nullptr;
-    PFN_vkGetPhysicalDeviceExternalBufferProperties vkGetPhysicalDeviceExternalBufferProperties = nullptr;
-    PFN_vkGetPhysicalDeviceExternalFenceProperties vkGetPhysicalDeviceExternalFenceProperties = nullptr;
-    PFN_vkGetPhysicalDeviceExternalSemaphoreProperties vkGetPhysicalDeviceExternalSemaphoreProperties = nullptr;
-
-    PFN_vkGetDeviceProcAddr vkGetDeviceProcAddr = nullptr;
-    PFN_vkCreateDevice vkCreateDevice = nullptr;
-
-    // WSI
-    PFN_vkCreateHeadlessSurfaceEXT vkCreateHeadlessSurfaceEXT = nullptr;
-    PFN_vkCreateDisplayPlaneSurfaceKHR vkCreateDisplayPlaneSurfaceKHR = nullptr;
-    PFN_vkGetPhysicalDeviceDisplayPropertiesKHR vkGetPhysicalDeviceDisplayPropertiesKHR = nullptr;
-    PFN_vkGetPhysicalDeviceDisplayPlanePropertiesKHR vkGetPhysicalDeviceDisplayPlanePropertiesKHR = nullptr;
-    PFN_vkGetDisplayPlaneSupportedDisplaysKHR vkGetDisplayPlaneSupportedDisplaysKHR = nullptr;
-    PFN_vkGetDisplayModePropertiesKHR vkGetDisplayModePropertiesKHR = nullptr;
-    PFN_vkCreateDisplayModeKHR vkCreateDisplayModeKHR = nullptr;
-    PFN_vkGetDisplayPlaneCapabilitiesKHR vkGetDisplayPlaneCapabilitiesKHR = nullptr;
-    PFN_vkGetPhysicalDevicePresentRectanglesKHR vkGetPhysicalDevicePresentRectanglesKHR = nullptr;
-    PFN_vkGetPhysicalDeviceDisplayProperties2KHR vkGetPhysicalDeviceDisplayProperties2KHR = nullptr;
-    PFN_vkGetPhysicalDeviceDisplayPlaneProperties2KHR vkGetPhysicalDeviceDisplayPlaneProperties2KHR = nullptr;
-    PFN_vkGetDisplayModeProperties2KHR vkGetDisplayModeProperties2KHR = nullptr;
-    PFN_vkGetDisplayPlaneCapabilities2KHR vkGetDisplayPlaneCapabilities2KHR = nullptr;
-    PFN_vkGetPhysicalDeviceSurfaceCapabilities2KHR vkGetPhysicalDeviceSurfaceCapabilities2KHR = nullptr;
-    PFN_vkGetPhysicalDeviceSurfaceFormats2KHR vkGetPhysicalDeviceSurfaceFormats2KHR = nullptr;
-
-#if defined(VK_USE_PLATFORM_ANDROID_KHR)
-    PFN_vkCreateAndroidSurfaceKHR vkCreateAndroidSurfaceKHR = nullptr;
-#endif  // VK_USE_PLATFORM_ANDROID_KHR
-#if defined(VK_USE_PLATFORM_DIRECTFB_EXT)
-    PFN_vkCreateDirectFBSurfaceEXT vkCreateDirectFBSurfaceEXT = nullptr;
-    PFN_vkGetPhysicalDeviceDirectFBPresentationSupportEXT vkGetPhysicalDeviceDirectFBPresentationSupportEXT = nullptr;
-#endif  // VK_USE_PLATFORM_DIRECTFB_EXT
-#if defined(VK_USE_PLATFORM_FUCHSIA)
-    PFN_vkCreateImagePipeSurfaceFUCHSIA vkCreateImagePipeSurfaceFUCHSIA = nullptr;
-#endif  // VK_USE_PLATFORM_FUCHSIA
-#if defined(VK_USE_PLATFORM_GGP)
-    PFN_vkCreateStreamDescriptorSurfaceGGP vkCreateStreamDescriptorSurfaceGGP = nullptr;
-#endif  // VK_USE_PLATFORM_GGP
-#if defined(VK_USE_PLATFORM_IOS_MVK)
-    PFN_vkCreateIOSSurfaceMVK vkCreateIOSSurfaceMVK = nullptr;
-#endif  // VK_USE_PLATFORM_IOS_MVK
-#if defined(VK_USE_PLATFORM_MACOS_MVK)
-    PFN_vkCreateMacOSSurfaceMVK vkCreateMacOSSurfaceMVK = nullptr;
-#endif  // VK_USE_PLATFORM_MACOS_MVK
-#if defined(VK_USE_PLATFORM_METAL_EXT)
-    PFN_vkCreateMetalSurfaceEXT vkCreateMetalSurfaceEXT = nullptr;
-#endif  // VK_USE_PLATFORM_METAL_EXT
-#if defined(VK_USE_PLATFORM_SCREEN_QNX)
-    PFN_vkCreateScreenSurfaceQNX vkCreateScreenSurfaceQNX = nullptr;
-    PFN_vkGetPhysicalDeviceScreenPresentationSupportQNX vkGetPhysicalDeviceScreenPresentationSupportQNX = nullptr;
-#endif  // VK_USE_PLATFORM_SCREEN_QNX
-#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
-    PFN_vkCreateWaylandSurfaceKHR vkCreateWaylandSurfaceKHR = nullptr;
-    PFN_vkGetPhysicalDeviceWaylandPresentationSupportKHR vkGetPhysicalDeviceWaylandPresentationSupportKHR = nullptr;
-#endif  // VK_USE_PLATFORM_WAYLAND_KHR
-#if defined(VK_USE_PLATFORM_XCB_KHR)
-    PFN_vkCreateXcbSurfaceKHR vkCreateXcbSurfaceKHR = nullptr;
-    PFN_vkGetPhysicalDeviceXcbPresentationSupportKHR vkGetPhysicalDeviceXcbPresentationSupportKHR = nullptr;
-#endif  // VK_USE_PLATFORM_XCB_KHR
-#if defined(VK_USE_PLATFORM_XLIB_KHR)
-    PFN_vkCreateXlibSurfaceKHR vkCreateXlibSurfaceKHR = nullptr;
-    PFN_vkGetPhysicalDeviceXlibPresentationSupportKHR vkGetPhysicalDeviceXlibPresentationSupportKHR = nullptr;
-#endif  // VK_USE_PLATFORM_XLIB_KHR
-#if defined(VK_USE_PLATFORM_WIN32_KHR)
-    PFN_vkCreateWin32SurfaceKHR vkCreateWin32SurfaceKHR = nullptr;
-    PFN_vkGetPhysicalDeviceWin32PresentationSupportKHR vkGetPhysicalDeviceWin32PresentationSupportKHR = nullptr;
-#endif  // VK_USE_PLATFORM_WIN32_KHR
-    PFN_vkDestroySurfaceKHR vkDestroySurfaceKHR = nullptr;
-
-    // instance extensions functions (can only be loaded with a valid instance)
-    PFN_vkCreateDebugUtilsMessengerEXT vkCreateDebugUtilsMessengerEXT = nullptr;    // Null unless the extension is enabled
-    PFN_vkDestroyDebugUtilsMessengerEXT vkDestroyDebugUtilsMessengerEXT = nullptr;  // Null unless the extension is enabled
-    PFN_vkCreateDebugReportCallbackEXT vkCreateDebugReportCallbackEXT = nullptr;    // Null unless the extension is enabled
-    PFN_vkDestroyDebugReportCallbackEXT vkDestroyDebugReportCallbackEXT = nullptr;  // Null unless the extension is enabled
-
-    // device functions
-    PFN_vkDestroyDevice vkDestroyDevice = nullptr;
-    PFN_vkGetDeviceQueue vkGetDeviceQueue = nullptr;
-
-    VulkanFunctions();
-
-    void load_instance_functions(VkInstance instance);
-
-    FromVoidStarFunc load(VkInstance inst, const char* func_name) const {
-        return FromVoidStarFunc(vkGetInstanceProcAddr(inst, func_name));
+template <typename T, size_t U>
+bool check_permutation(std::initializer_list<const char*> expected, std::array<T, U> const& returned) {
+    if (expected.size() != returned.size()) return false;
+    for (uint32_t i = 0; i < expected.size(); i++) {
+        auto found = std::find_if(std::begin(returned), std::end(returned),
+                                  [&](T elem) { return string_eq(*(expected.begin() + i), elem.layerName); });
+        if (found == std::end(returned)) return false;
     }
-
-    FromVoidStarFunc load(VkDevice device, const char* func_name) const {
-        return FromVoidStarFunc(vkGetDeviceProcAddr(device, func_name));
+    return true;
+}
+template <typename T>
+bool check_permutation(std::initializer_list<const char*> expected, std::vector<T> const& returned) {
+    if (expected.size() != returned.size()) return false;
+    for (uint32_t i = 0; i < expected.size(); i++) {
+        auto found = std::find_if(std::begin(returned), std::end(returned),
+                                  [&](T elem) { return string_eq(*(expected.begin() + i), elem.layerName); });
+        if (found == std::end(returned)) return false;
     }
-};
-
-struct DeviceFunctions {
-    PFN_vkGetDeviceProcAddr vkGetDeviceProcAddr = nullptr;
-    PFN_vkDestroyDevice vkDestroyDevice = nullptr;
-    PFN_vkGetDeviceQueue vkGetDeviceQueue = nullptr;
-    PFN_vkCreateCommandPool vkCreateCommandPool = nullptr;
-    PFN_vkAllocateCommandBuffers vkAllocateCommandBuffers = nullptr;
-    PFN_vkDestroyCommandPool vkDestroyCommandPool = nullptr;
-    PFN_vkCreateSwapchainKHR vkCreateSwapchainKHR = nullptr;
-    PFN_vkGetSwapchainImagesKHR vkGetSwapchainImagesKHR = nullptr;
-    PFN_vkDestroySwapchainKHR vkDestroySwapchainKHR = nullptr;
-
-    DeviceFunctions() = default;
-    DeviceFunctions(const VulkanFunctions& vulkan_functions, VkDevice device);
-
-    FromVoidStarFunc load(VkDevice device, const char* func_name) const {
-        return FromVoidStarFunc(vkGetDeviceProcAddr(device, func_name));
-    }
-};
+    return true;
+}
 
 // InstWrapper & DeviceWrapper - used to make creating instances & devices easier when writing tests
 struct InstWrapper {
@@ -478,11 +339,12 @@ void FillDebugUtilsCreateDetails(InstanceCreateInfo& create_info, DebugUtilsLogg
 void FillDebugUtilsCreateDetails(InstanceCreateInfo& create_info, DebugUtilsWrapper& wrapper);
 
 struct LoaderSettingsLayerConfiguration {
-    BUILDER_VALUE(LoaderSettingsLayerConfiguration, std::string, name, {})
-    BUILDER_VALUE(LoaderSettingsLayerConfiguration, std::filesystem::path, path, {})
-    BUILDER_VALUE(LoaderSettingsLayerConfiguration, std::string, control, {})
-    BUILDER_VALUE(LoaderSettingsLayerConfiguration, bool, treat_as_implicit_manifest, false)
+    BUILDER_VALUE(std::string, name)
+    BUILDER_VALUE(std::filesystem::path, path)
+    BUILDER_VALUE(std::string, control)
+    BUILDER_VALUE(bool, treat_as_implicit_manifest)
 };
+// Needed for next_permutation
 inline bool operator==(LoaderSettingsLayerConfiguration const& a, LoaderSettingsLayerConfiguration const& b) {
     return a.name == b.name && a.path == b.path && a.control == b.control &&
            a.treat_as_implicit_manifest == b.treat_as_implicit_manifest;
@@ -495,28 +357,44 @@ inline bool operator>(LoaderSettingsLayerConfiguration const& a, LoaderSettingsL
 inline bool operator<=(LoaderSettingsLayerConfiguration const& a, LoaderSettingsLayerConfiguration const& b) { return !(b < a); }
 inline bool operator>=(LoaderSettingsLayerConfiguration const& a, LoaderSettingsLayerConfiguration const& b) { return !(a < b); }
 
+struct LoaderSettingsDriverConfiguration {
+    BUILDER_VALUE(std::filesystem::path, path)
+};
+
+using VulkanUUID = std::array<uint8_t, VK_UUID_SIZE>;
+
+struct LoaderSettingsDeviceConfiguration {
+    BUILDER_VALUE(VulkanUUID, deviceUUID)
+    BUILDER_VALUE(VulkanUUID, driverUUID)
+    BUILDER_VALUE(uint32_t, driverVersion)
+    BUILDER_VALUE(std::string, deviceName)
+    BUILDER_VALUE(std::string, driverName)
+};
+
 // Log files and their associated filter
 struct LoaderLogConfiguration {
-    BUILDER_VECTOR(LoaderLogConfiguration, std::string, destinations, destination)
-    BUILDER_VECTOR(LoaderLogConfiguration, std::string, filters, filter)
+    BUILDER_VECTOR(std::string, destinations, destination)
+    BUILDER_VECTOR(std::string, filters, filter)
 };
 struct AppSpecificSettings {
-    BUILDER_VECTOR(AppSpecificSettings, std::string, app_keys, app_key)
-    BUILDER_VECTOR(AppSpecificSettings, LoaderSettingsLayerConfiguration, layer_configurations, layer_configuration)
-    BUILDER_VECTOR(AppSpecificSettings, std::string, stderr_log, stderr_log_filter)
-    BUILDER_VECTOR(AppSpecificSettings, LoaderLogConfiguration, log_configurations, log_configuration)
+    BUILDER_VECTOR(std::string, app_keys, app_key)
+    BUILDER_VECTOR(LoaderSettingsLayerConfiguration, layer_configurations, layer_configuration)
+    BUILDER_VECTOR(LoaderSettingsDriverConfiguration, driver_configurations, driver_configuration)
+    BUILDER_VECTOR(LoaderSettingsDeviceConfiguration, device_configurations, device_configuration)
+    BUILDER_VALUE(bool, additional_drivers_use_exclusively)
+    BUILDER_VECTOR(std::string, stderr_log, stderr_log_filter)
+    BUILDER_VECTOR(LoaderLogConfiguration, log_configurations, log_configuration)
 };
 
 struct LoaderSettings {
-    BUILDER_VALUE(LoaderSettings, ManifestVersion, file_format_version, {})
-    BUILDER_VECTOR(LoaderSettings, AppSpecificSettings, app_specific_settings, app_specific_setting);
+    BUILDER_VALUE(ManifestVersion, file_format_version)
+    BUILDER_VECTOR(AppSpecificSettings, app_specific_settings, app_specific_setting);
 };
 
 struct FrameworkEnvironment;  // forward declaration
 
 struct PlatformShimWrapper {
-    PlatformShimWrapper(std::vector<fs::FolderManager>* folders, const char* log_filter) noexcept;
-    ~PlatformShimWrapper() noexcept;
+    PlatformShimWrapper(fs::FileSystemManager& file_system_manager, const char* log_filter) noexcept;
     PlatformShimWrapper(PlatformShimWrapper const&) = delete;
     PlatformShimWrapper& operator=(PlatformShimWrapper const&) = delete;
 
@@ -565,77 +443,47 @@ struct TestLayerHandle {
         shimmed_manifest_path;  // path to where the loader will find the manifest file (eg /usr/local/share/vulkan/<...>)
 };
 
-// Controls whether to create a manifest and where to put it
-enum class ManifestDiscoveryType {
-    generic,              // put the manifest in the regular locations
-    unsecured_generic,    // put the manifest in a user folder rather than system
-    none,                 // Do not write the manifest anywhere (for Direct Driver Loading)
-    null_dir,             // put the manifest in the 'null_dir' which the loader does not search in (D3DKMT for instance)
-    env_var,              // use the corresponding env-var for it
-    add_env_var,          // use the corresponding add-env-var for it
-    override_folder,      // add to a special folder for the override layer to use
-    windows_app_package,  // let the app package search find it
-    macos_bundle,         // place it in a location only accessible to macos bundles
-};
-
-enum class LibraryPathType {
-    absolute,              // default for testing - the exact path of the binary
-    relative,              // Relative to the manifest file
-    default_search_paths,  // Dont add any path information to the library_path - force the use of the default search paths
-};
-
 struct TestICDDetails {
     TestICDDetails(ManifestICD icd_manifest) noexcept : icd_manifest(icd_manifest) {}
     TestICDDetails(std::filesystem::path icd_binary_path, uint32_t api_version = VK_API_VERSION_1_0) noexcept {
         icd_manifest.set_lib_path(icd_binary_path).set_api_version(api_version);
     }
-    BUILDER_VALUE(TestICDDetails, ManifestICD, icd_manifest, {});
-    BUILDER_VALUE(TestICDDetails, std::filesystem::path, json_name, "test_icd");
+    BUILDER_VALUE(ManifestICD, icd_manifest);
+    BUILDER_VALUE_WITH_DEFAULT(std::filesystem::path, json_name, "test_icd");
     // Uses the json_name without modification - default is to append _1 in the json file to distinguish drivers
-    BUILDER_VALUE(TestICDDetails, bool, disable_icd_inc, false);
-    BUILDER_VALUE(TestICDDetails, ManifestDiscoveryType, discovery_type, ManifestDiscoveryType::generic);
-    BUILDER_VALUE(TestICDDetails, bool, is_fake, false);
+    BUILDER_VALUE(bool, disable_icd_inc);
+    BUILDER_VALUE_WITH_DEFAULT(ManifestDiscoveryType, discovery_type, ManifestDiscoveryType::generic);
+    BUILDER_VALUE(bool, is_fake);
     // If discovery type is env-var, is_dir controls whether to use the path to the file or folder the manifest is in
-    BUILDER_VALUE(TestICDDetails, bool, is_dir, false);
-    BUILDER_VALUE(TestICDDetails, LibraryPathType, library_path_type, LibraryPathType::absolute);
+    BUILDER_VALUE(bool, is_dir);
+    BUILDER_VALUE_WITH_DEFAULT(LibraryPathType, library_path_type, LibraryPathType::absolute);
 };
 
 struct TestLayerDetails {
     TestLayerDetails(ManifestLayer layer_manifest, const std::string& json_name) noexcept
         : layer_manifest(layer_manifest), json_name(json_name) {}
-    BUILDER_VALUE(TestLayerDetails, ManifestLayer, layer_manifest, {});
-    BUILDER_VALUE(TestLayerDetails, std::string, json_name, "test_layer");
-    BUILDER_VALUE(TestLayerDetails, ManifestDiscoveryType, discovery_type, ManifestDiscoveryType::generic);
-    BUILDER_VALUE(TestLayerDetails, bool, is_fake, false);
+    BUILDER_VALUE(ManifestLayer, layer_manifest);
+    BUILDER_VALUE_WITH_DEFAULT(std::string, json_name, "test_layer");
+    BUILDER_VALUE_WITH_DEFAULT(ManifestDiscoveryType, discovery_type, ManifestDiscoveryType::generic);
+    BUILDER_VALUE(bool, is_fake);
     // If discovery type is env-var, is_dir controls whether to use the path to the file or folder the manifest is in
-    BUILDER_VALUE(TestLayerDetails, bool, is_dir, true);
-    BUILDER_VALUE(TestLayerDetails, LibraryPathType, library_path_type, LibraryPathType::absolute);
-};
-
-// Locations manifests can go in the test framework
-// If this enum is added to - the contructor of FrameworkEnvironment also needs to be updated with the new enum value
-enum class ManifestLocation {
-    null = 0,
-    driver = 1,
-    driver_env_var = 2,
-    explicit_layer = 3,
-    explicit_layer_env_var = 4,
-    explicit_layer_add_env_var = 5,
-    implicit_layer = 6,
-    implicit_layer_env_var = 7,
-    implicit_layer_add_env_var = 8,
-    override_layer = 9,
-    windows_app_package = 10,
-    macos_bundle = 11,
-    unsecured_location = 12,
-    settings_location = 13,
+    BUILDER_VALUE_WITH_DEFAULT(bool, is_dir, true);
+    BUILDER_VALUE_WITH_DEFAULT(LibraryPathType, library_path_type, LibraryPathType::absolute);
 };
 
 struct FrameworkSettings {
-    BUILDER_VALUE(FrameworkSettings, const char*, log_filter, "all");
-    BUILDER_VALUE(FrameworkSettings, bool, enable_default_search_paths, true);
-    BUILDER_VALUE(FrameworkSettings, LoaderSettings, loader_settings, {});
-    BUILDER_VALUE(FrameworkSettings, bool, secure_loader_settings, false);
+    BUILDER_VALUE_WITH_DEFAULT(const char*, log_filter, "all");
+    BUILDER_VALUE_WITH_DEFAULT(bool, run_as_if_with_elevated_privleges, false);
+
+#if TESTING_COMMON_UNIX_PLATFORMS
+    BUILDER_VALUE_WITH_DEFAULT(std::string, home_env_var, "/home/fake_home");
+#if !defined(__APPLE__)
+    BUILDER_VALUE(std::string, xdg_config_home_env_var);
+    BUILDER_VALUE(std::string, xdg_config_dirs_env_var);
+    BUILDER_VALUE(std::string, xdg_data_home_env_var);
+    BUILDER_VALUE(std::string, xdg_data_dirs_env_var);
+#endif
+#endif
 };
 
 struct FrameworkEnvironment {
@@ -654,12 +502,20 @@ struct FrameworkEnvironment {
     void add_fake_implicit_layer(ManifestLayer layer_manifest, const std::string& json_name) noexcept;
     void add_fake_explicit_layer(ManifestLayer layer_manifest, const std::string& json_name) noexcept;
 
-    // resets the current settings with the values contained in loader_settings
-    void write_settings_file(std::string const& file_contents);
-    // apply any changes made to FrameworkEnvironment's loader_settings member
-    void update_loader_settings(const LoaderSettings& loader_settings) noexcept;
+    // Resets the current settings with the values contained in loader_settings.
+    // Write_to_secure_location determines whether to write to the secure or unsecure settings folder.
+    void write_settings_file(std::string const& file_contents, bool write_to_secure_location);
+
+    // Apply any changes made to FrameworkEnvironment's loader_settings member.
+    // By default writes to the secure settings location
+    void update_loader_settings(const LoaderSettings& loader_settings, bool write_to_secure_location = true) noexcept;
+
     void remove_loader_settings();
 
+    // Creates a file called `file_name` for the given `category` in the given `location` with `source_string` as the contents
+    void write_file_from_string(std::string const& source_string, ManifestCategory category, ManifestLocation location,
+                                std::string const& file_name);
+    // Creates a file called `file_name` for the given `category` in the given `location` with contents copied from `source_file`
     void write_file_from_source(const char* source_file, ManifestCategory category, ManifestLocation location,
                                 std::string const& file_name);
 
@@ -675,15 +531,19 @@ struct FrameworkEnvironment {
     std::filesystem::path get_layer_manifest_path(size_t index = 0) noexcept;
     std::filesystem::path get_shimmed_layer_manifest_path(size_t index = 0) noexcept;
 
-    fs::FolderManager& get_folder(ManifestLocation location) noexcept;
-    fs::FolderManager const& get_folder(ManifestLocation location) const noexcept;
+    fs::Folder& get_folder(ManifestLocation location) noexcept;
+    fs::Folder const& get_folder(ManifestLocation location) const noexcept;
 #if defined(__APPLE__)
     // Set the path of the app bundle to the appropriate test framework bundle
     void setup_macos_bundle() noexcept;
 #endif
 
+    void add_symlink(ManifestLocation location, std::filesystem::path const& target, std::filesystem::path const& link_name);
+
     FrameworkSettings settings;
 
+    fs::FileSystemManager file_system_manager;
+
     // Query the global extensions
     // Optional: use layer_name to query the extensions of a specific layer
     std::vector<VkExtensionProperties> GetInstanceExtensions(uint32_t count, const char* layer_name = nullptr);
@@ -691,20 +551,34 @@ struct FrameworkEnvironment {
     std::vector<VkLayerProperties> GetLayerProperties(uint32_t count);
 
     PlatformShimWrapper platform_shim;
-    std::vector<fs::FolderManager> folders;
-
-    DebugUtilsLogger debug_log;
-    VulkanFunctions vulkan_functions;
 
     std::vector<TestICDHandle> icds;
     std::vector<TestLayerHandle> layers;
 
+    DebugUtilsLogger debug_log;
+    VulkanFunctions vulkan_functions;
+
     EnvVarWrapper env_var_vk_icd_filenames{"VK_DRIVER_FILES"};
     EnvVarWrapper add_env_var_vk_icd_filenames{"VK_ADD_DRIVER_FILES"};
     EnvVarWrapper env_var_vk_layer_paths{"VK_LAYER_PATH"};
     EnvVarWrapper add_env_var_vk_layer_paths{"VK_ADD_LAYER_PATH"};
     EnvVarWrapper env_var_vk_implicit_layer_paths{"VK_IMPLICIT_LAYER_PATH"};
     EnvVarWrapper add_env_var_vk_implicit_layer_paths{"VK_ADD_IMPLICIT_LAYER_PATH"};
+    EnvVarWrapper env_var_vk_loader_device_id_filter{"VK_LOADER_DEVICE_ID_FILTER"};
+    EnvVarWrapper env_var_vk_loader_vendor_id_filter{"VK_LOADER_VENDOR_ID_FILTER"};
+    EnvVarWrapper env_var_vk_loader_driver_id_filter{"VK_LOADER_DRIVER_ID_FILTER"};
+
+#if TESTING_COMMON_UNIX_PLATFORMS
+    EnvVarWrapper env_var_home{"HOME", "/home/fake_home"};
+#if !defined(__APPLE__)
+    EnvVarWrapper env_var_xdg_config_home{"XDG_CONFIG_HOME"};
+    EnvVarWrapper env_var_xdg_config_dirs{"XDG_CONFIG_DIRS"};
+    EnvVarWrapper env_var_xdg_data_home{"XDG_DATA_HOME"};
+    EnvVarWrapper env_var_xdg_data_dirs{"XDG_DATA_DIRS"};
+#endif
+    std::string secure_manifest_base_location;
+    std::string unsecure_manifest_base_location;
+#endif
 
     LoaderSettings loader_settings;  // the current settings written to disk
    private:
